<?php
namespace Geral\Entities;

use \Geral\Entities\Paramfilial;

/**
 * @method \Geral\Repositories\Filial getRepository static
 * @method \Geral\Entities\Filial findOneByCdfilialAndNrorg($cdfilial, $nrorg) static
 * @method \Geral\Entities\Filial findOneByNrorgAndCdregiao($nrorg, $regiao) static
 */
class Filial extends Base\Filial
{

    const TIPO_INSCRICAO_FISICA = 'F';

    const TIPO_INSCRICAO_CGC = 'J';

    const TIPO_INSCRICAO_JURIDICA = 'I';

    const TIPO_INSCRICAO_OUTROS = 'O';

    /**
     * Constante usada para as filiais que nao utilizam almoxarifado padrao.
     */
    const SEM_ALMOXARIFADO_PADRAO = " ";
    /**
     * Constantes para as filiais que nao utilizam estoque padrao.
     */
    const SEM_LOCAL_ESTOQUE_PADRAO = " ";

    /**
     * Constante para o valor de utiliza Almoxarifado igual a sim, para o campo de IDUTILALMOX
     */
    const UTILIZA_ALMOXARIFADO = "S";
    /**
     * Constante para o valor de utiliza Almoxarifado igual a nao, para o campo de IDUTILALMOX
     */
    const NAO_UTILIZA_ALMOXARIFADO = "N";

    /**
     * Constante para o valor de utiliza Almoxarifado igual a sim, para o campo de IDUTILLCESTQ
     */
    const UTILIZA_ESTOQUE_PADRAO = "S";
    /**
     * Constante para o valor de utiliza Almoxarifado igual a nao, para o campo de IDUTILLCESTQ
     */
    const NAO_UTILIZA_ESTOQUE_PADRAO = "N";

    /**
     * IDCONSORCFIL == S: Consiste orcamento da filial.
     */
    const CONSISTE_ORCAMENTO = "S";
    /**
     * IDCONSORCFIL == N: Nao consiste orcamento da filial.
     */
    const NAO_CONSISTE_ORCAMENTO = "N";

    /**
     * Identificador para filiais com contrato rescindido.
     */
    const CONTRATO_RESCINDIDO = "S";
    /**
     * Identificador para filiais de contrato nao rescindido.
     */
    const CONTRATO_NAO_RESCINDIDO = "N";

    /** @var \Geral\Entities\Endefili */
    protected $endefiliprinc = null;

    public function __construct()
    {
        $this->idativo         = 'S';
        $this->dtinclusao      = new \DateTime();
        $this->idutfilipadr    = 'S';
        $this->idcomparqxmlfil = 'S';
        $this->idconsorcfil    = 'N';
    }

    /**
     * @return \Geral\Entities\Endefili
     */
    public function getEndefiliprinc()
    {
        if ($this->endefiliprinc === null) {
            $id                  = array(
                "idtpendefili" => 'P',
                "cdfilial"     => $this->cdfilial,
                "nrorg"        => $this->nrorg,
            );
            $this->endefiliprinc = \Geral\Entities\Endefili::getObject($id);
        }

        return $this->endefiliprinc;
    }

    /**
     * Retorna o almoxarifado padrao da filial. Caso esta nao utilize almoxarifado padrao, deve retornar o valor
     * reservado para as filiais sem almoxarifado.
     *
     * @return string
     */
    public function getAlmoxarifadoPadrao()
    {
        /** @var $paramFilial Paramfilial */
        $paramFilial = Paramfilial::findOneByCdfilialAndNrorg(
            $this->getCdfilial(), $this->getNrorg()
        );

        $almoxarifadoPadrao  = $paramFilial->getCdalmoxarife();
        $utilizaAlmoxarifado = $paramFilial->getIdutilalmox();

        if ($utilizaAlmoxarifado === self::UTILIZA_ALMOXARIFADO) {
            return $almoxarifadoPadrao;
        } else {
            return self::SEM_ALMOXARIFADO_PADRAO;
        }
    }

    /**
     * Retorna o estoque padrao da filial. Caso esta nao utilize estoque padrao, deve retornar o valor
     * reservado para as filiais sem estoque.
     *
     * @return string
     */
    public function getEstoquePadrao()
    {
        /** @var $paramFilial Paramfilial */
        $paramFilial = Paramfilial::findOneByCdfilialAndNrorg(
            $this->getCdfilial(), $this->getNrorg()
        );

        $estoquePadrao  = $paramFilial->getCdlocalestoq();
        $utilizaEstoque = $paramFilial->getIdutillcestq();

        if ($utilizaEstoque === self::UTILIZA_ESTOQUE_PADRAO) {
            return $estoquePadrao;
        } else {
            return self::SEM_LOCAL_ESTOQUE_PADRAO;
        }
    }

    /**
     * Verifica se a filial utiliza Orcamento GOP e se a filial nao esta com o contrato rescindido.
     *
     * @return bool
     */
    public function utilizaOrcamentoGOP()
    {
        // Caso nao tenha nenhum valor setado para este parametro, considera-se que a filial nao consiste orcamento.
        $param = $this->getIdconsorcfil() ? $this->getIdconsorcfil()
            : self::NAO_CONSISTE_ORCAMENTO;

        // Retorna true se a filial utiliza orcamento GOP e se esta nao esta com o contrato rescindido.
        return $param === self::CONSISTE_ORCAMENTO
        && $this->getIdcntrresc() === self::CONTRATO_NAO_RESCINDIDO;
    }

    /**
     * Retorna a filial usada como central de compras. Caso o atributo CDFILCENTCMP esteja vazio (ou seja, a filial
     * corrente é uma central de compras), o método deve retornar o codigo da filial (CDFILIAL).
     *
     * @return String
     */
    public function getCentralDeCompras()
    {
        if ($this->getCdfilcentcmp()) {
            return $this->getCdfilcentcmp();
        }

        return $this->getCdfilial();
    }

    public static function retornaFilial($cdfilial, $nrorg)
    {

        $retorno = self::getObjectByWhere(
            'CDFILIAL = ? AND NRORG = ?', array($cdfilial, $nrorg)
        );

        return $retorno;

        if ($retorno instanceof Filial) {
            return $result;
        }

        throw new \Exception('FilialEntity: Nao existe esta filial!');
    }

    //end retornaFilial()

    public static function retornaFiliaisAgregadas($nrorg, $central)
    {
        try {
            $agregadas = self::getObjectsByWhere(
                'NRORG = ? AND CDFILCENTPME = ?',
                array($nrorg, $central)
            );

            return $agregadas;
        } catch (\Exception $e) {
            throw $e;
        }
    }

    /**
     * Retorna array de objetos da classe Filial
     *
     * @static
     * @return mixed
     */
    public static function retornaObjsFilial()
    {
        $array_Filial = self::getObjectsByWhere('1 = 1');
        return $array_Filial;
    }
}